package com.m2pool.common.core.utils;

import com.m2pool.common.core.constant.OpenApiKeyConstants;
import com.m2pool.common.core.constant.TokenConstants;
import com.m2pool.common.core.text.Convert;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

import java.util.Map;

/**
 * @Description Jwt工具类
 * @Date 2024/6/12 16:38
 * @Author dy
 */
public class OpenApiJwtUtils
{
    public static String secret = TokenConstants.API_SECRET;

    /**
     * 从数据声明生成令牌
     *
     * @param claims 数据声明
     * @return 令牌
     */
    public static String createApiKey(Map<String, Object> claims)
    {
        String apiKey = Jwts.builder().setClaims(claims).signWith(SignatureAlgorithm.HS512, secret).compact();
        return apiKey;
    }

    /**
     * 从令牌中获取数据声明
     *
     * @param apiKey 令牌
     * @return 数据声明
     */
    public static Claims parseApiKey(String apiKey)
    {
        Claims claims = null;
        try {
            claims = Jwts.parser().setSigningKey(secret).parseClaimsJws(apiKey).getBody();
        }catch (Exception e){
            System.out.println("Jwt解析失败");
        }
        return claims;
    }

    /**
     * 根据apiKey获取apiKey
     *
     * @param apiKey 令牌
     * @return apiKey
     */
    public static String getApiKey(String apiKey)
    {
        Claims claims = parseApiKey(apiKey);
        return getValue(claims, OpenApiKeyConstants.API_KEY);
    }

    /**
     * 根据apiKey获取apiKey
     *
     * @return apiKey
     */
    public static String getApiKey(Claims claims)
    {
        return getValue(claims, OpenApiKeyConstants.API_KEY);
    }

    /**
     * 根据apiKey获取用户邮箱
     *
     * @param apiKey 令牌
     * @return 用户邮箱
     */
    public static String getApiUser(String apiKey)
    {
        Claims claims = parseApiKey(apiKey);
        return getValue(claims, OpenApiKeyConstants.API_USER);
    }

    /**
     * 根据身份信息获取用户邮箱
     *
     * @param claims 身份信息
     * @return 用户有
     */
    public static String getApiUser(Claims claims)
    {
        return getValue(claims, OpenApiKeyConstants.API_USER);
    }

    /**
     * 根据apiKey获取用户绑定ip
     *
     * @param apiKey 令牌
     * @return 用户绑定ip
     */
    public static String getApiIp(String apiKey)
    {
        Claims claims = parseApiKey(apiKey);
        return getValue(claims, OpenApiKeyConstants.API_IP);
    }

    /**
     * 根据apiKey获取用户绑定ip
     *
     * @param claims 身份信息
     * @return 用户绑定ip
     */
    public static String getApiIp(Claims claims)
    {
        return getValue(claims, OpenApiKeyConstants.API_IP);
    }


    /**
     * 根据身份信息获取用户绑定ip
     *
     * @param claims 身份信息
     * @return 用户绑定ip
     */
    public static String getUserName(Claims claims)
    {
        return getValue(claims, OpenApiKeyConstants.API_IP);
    }

    /**
     * 根据身份信息获取键值
     *
     * @param claims 身份信息
     * @param key 键
     * @return 值
     */
    public static String getValue(Claims claims, String key)
    {
        return Convert.toStr(claims.get(key), "");
    }
}
