package com.m2pool.chat.controller;

import com.m2pool.chat.dto.WebsocketMessageDto;
import com.m2pool.chat.service.ChatService;
import com.m2pool.chat.vo.ChatHistoryVo;
import com.m2pool.chat.vo.UserMessageVo;
import com.m2pool.common.core.utils.StringUtils;
import com.m2pool.common.core.web.Result.AjaxResult;
import com.m2pool.common.core.web.controller.BaseController;
import org.springframework.messaging.handler.annotation.DestinationVariable;
import org.springframework.messaging.handler.annotation.MessageMapping;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.messaging.simp.annotation.SendToUser;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;

/**
 * @Description TODO
 * @Date 2025/2/24 15:05
 * @Author 杜懿
 */
@Controller
public class ChatController extends BaseController {

    @Resource
    private ChatService chatService;

    //@Resource
    //    //private WebSocketServer webSocketServer;

    @GetMapping("/history")
    @ResponseBody
    public AjaxResult getChatHistory(@RequestBody ChatHistoryVo vo) {

        if(StringUtils.isNull(vo)){
            return AjaxResult.error("请求参数为空");
        }

        String identifier = StringUtils.isNotBlank(vo.getEmail()) ? vo.getEmail() : vo.getBrowserId();
        return chatService.getHistory(identifier);
    }

    //spring提供的推送方式
    @Resource
    private SimpMessagingTemplate messagingTemplate;


    /**
     * 发送消息到对应的用户
     * @param userId 用户id，消息接受者
     * @param userMessageVo 消息体
     * @return 返回值通过CommonMessageConvert消息转换器转换
     */
    @MessageMapping("/message/{userId}")
    @SendToUser("/queue/{userId}")
    public WebsocketMessageDto sendMessageToUser(@DestinationVariable String userId, UserMessageVo userMessageVo) {
        WebsocketMessageDto websocketMessageDto = new WebsocketMessageDto();
        websocketMessageDto.setType("message");
        websocketMessageDto.setMessage(userMessageVo.getMessage());

        return websocketMessageDto;
    }


    //TODO 前端打开聊天框，获取用户信息，建立一对一链接


    //TODO 用户登录后，保存历史信息到数据库表,分表存储(7天)。


    //TODO 用户注销，需删除历史信息


    //



}
