package com.m2pool.chat.interceptor;


import org.springframework.lang.Nullable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.simp.stomp.StompCommand;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.messaging.support.ChannelInterceptor;

/**
 * @ClassName WebsocketChannelInterceptor
 * @Description websocket channel 通道拦截器
 * @Author yyb
 * @Date 2025/4/10 15:44
 */
public class WebsocketChannelInterceptor implements ChannelInterceptor {
    private static final Logger LOGGER = LoggerFactory.getLogger(WebsocketChannelInterceptor.class);

    /**
     * websocket channel 通道拦截器
     * @param message
     * @param channel
     * @return
     */
    @Override
    public Message<?> preSend(Message<?> message, MessageChannel channel) {
        StompHeaderAccessor accessor = StompHeaderAccessor.wrap(message);

        //TODO 前端通过@stomp/stompjs、基于stomp-client+webscoket等插件实现，可以在该方法获取自定义请求头，做一些校验

        if (accessor.getCommand() == StompCommand.CONNECT) {
            LOGGER.info("------------收到websocket的连接消息");
        }

        if (accessor.getCommand() == StompCommand.SEND) {
            LOGGER.info("------------收到websocket的数据发送消息");
        }

        if (accessor.getCommand() == StompCommand.SUBSCRIBE) {
            LOGGER.info("------------收到websocket的订阅消息");
        }

        if (accessor.getCommand() == StompCommand.UNSUBSCRIBE) {
            LOGGER.info("------------收到websocket的取消订阅消息");
        }

        return message;
    }


    @Override
    public void postSend(Message<?> message, MessageChannel channel, boolean sent) {
        LOGGER.info("------------WebsocketChannelInterceptor-postSend");
    }

    @Override
    public void afterSendCompletion(Message<?> message, MessageChannel channel, boolean sent, @Nullable Exception ex) {
        LOGGER.info("-----------WebsocketChannelInterceptor-afterSendCompletion");
    }

    @Override
    public boolean preReceive(MessageChannel channel) {
        LOGGER.info("----------WebsocketChannelInterceptor-preReceive");
        return true;
    }

    @Override
    public Message<?> postReceive(Message<?> message, MessageChannel channel) {
        LOGGER.info("----------WebsocketChannelInterceptor-postReceive");
        return message;
    }

    @Override
    public void afterReceiveCompletion(@Nullable Message<?> message, MessageChannel channel, @Nullable Exception ex) {
        LOGGER.info("----------WebsocketChannelInterceptor-afterReceiveCompletion");
    }
}
