package com.m2pool.system.mapper;

import com.m2pool.common.datasource.annotation.Master;
import com.m2pool.system.api.entity.SysUser;
import com.m2pool.system.entity.GoogleInfo;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * @Description TODO
 * @Date 2024/6/14 10:37
 * @Author dy
 */
@Master
public interface SysUserMapper {

    /**
     * 根据条件分页查询用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUserList(SysUser sysUser);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    public SysUser selectUserByUserName(String userName);

    /**
     * 通过邮箱查询用户
     *
     * @param email 邮箱
     * @return 用户对象信息
     */
    public SysUser selectUserByEmail(String email);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public SysUser selectUserById(Long userId);

    /**
     * 校验用户名称是否唯一
     *
     * @param userName 用户名称
     * @return 结果
     */
    public int checkUserNameUnique(String userName);

    /**
     * 校验手机号码是否唯一
     *
     * @param phonenumber 手机号码
     * @return 结果
     */
    public SysUser checkPhoneUnique(String phonenumber);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @return 结果
     */
    public SysUser checkEmailUnique(String email);

    /**
     * 校验email是否存在
     *
     * @param email 用户邮箱
     * @return 结果
     */
    public String checkEmailExist(String email);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int insertUser(SysUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUser(SysUser user);


    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    public int resetUserPwd(@Param("userName") String userName, @Param("password") String password);


    /**
     * 重置用户密码
     *
     * @param email 邮箱
     * @param password 密码
     * @return 结果
     */
    public int resetPwdByEmail(@Param("email") String email, @Param("password") String password);


    @MapKey("user_id")
    public Map<Long, String> getUserInfoMap();

    /**
     * 获取谷歌验证码
     * @param email
     * @return
     */
    public GoogleInfo getGoogleInfoByEmail(@Param("email") String email);


}
